import 'package:core_plugin_flutter/core-plugin.dart';
import 'package:dio/dio.dart';
import 'ResultObject.dart';

enum MethodDio { GET, POST }

class CommonAppApi {
  static final CommonAppApi _singleton = CommonAppApi._internal();
  CommonAppApi._internal();
  factory CommonAppApi() => _singleton;
  /**
   * function public
   */
  Future<ResultObject> loadDataJsonResult(String url,
      {String baseUrl = '',
      MethodDio method = MethodDio.POST,
      Map<String, dynamic> queryParamater,
      Map<String, dynamic> headers,
      dynamic data}) async {
    try {
      var response = await loadDataJson(url,
          method: method,
          queryParamater: queryParamater,
          headers: headers,
          data: data);
      if (response.ErrorData == null && response.ErrorException == null)
        return ResultObject.fromJson(response.ResponseData.data);
      else
        return ResultObject(
            IsError: true,
            exception: response.ErrorData ?? response.ErrorException,
            ErrorMessage: response.ErrorData != null
                ? _handleError(response.ErrorData)
                : response.ErrorException.toString());
    } catch (error) {
      var result = new ResultObject();
      result.ErrorMessage =
          "Error loadDataJsonResult(url: $url, methodGet :${EnumToString.parse(method)} methodGet,queryParamater: $queryParamater  ) ${error.toString()}";
      result.IsError = true;
      return result;
    }
  }

/**
 * function private
 */
  Future<CommonResponse> loadDataJson(String url,
      {String baseUrl = '',
      MethodDio method = MethodDio.POST,
      Map<String, dynamic> queryParamater,
      Map<String, dynamic> headers,
      dynamic data}) async {
    Dio dio;
    try {
      BaseOptions options = BaseOptions(
        baseUrl: baseUrl,
        receiveTimeout: 5000,
        connectTimeout: 5000,
        responseType: ResponseType.json,
        contentType: Headers.jsonContentType,
      );
      if (headers != null) options.headers = headers;
      dio = Dio(options);
      dio.interceptors.clear();
      dio.interceptors.add(LoggingInterceptor());
      Response response = method == MethodDio.GET
          ? await dio.get(url, queryParameters: queryParamater)
          : await dio.post(url, data: data);

      return CommonResponse(ResponseData: response);
    } on DioError catch (ex) {
      return CommonResponse(ErrorData: ex);
    } catch (ex) {
      return CommonResponse(ErrorException: ex);
    } finally {
      dio.clear();
      dio.close(force: true);
    }
  }

  String _handleError(DioError dioError) {
    String errorDescription = "";
    switch (dioError.type) {
      case DioErrorType.CANCEL:
        errorDescription = "Request to API server was cancelled";
        break;
      case DioErrorType.CONNECT_TIMEOUT:
        errorDescription = "Connection timeout with API server";
        break;
      case DioErrorType.DEFAULT:
        errorDescription =
            "Connection to API server failed due to internet connection";
        break;
      case DioErrorType.RECEIVE_TIMEOUT:
        errorDescription = "Receive timeout in connection with API server";
        break;
      case DioErrorType.RESPONSE:
        errorDescription =
            "Received invalid status code: ${dioError.response.statusCode}";
        break;
      case DioErrorType.SEND_TIMEOUT:
        errorDescription = "Send timeout in connection with API server";
        break;
    }
    return errorDescription;
  }
}

class LoggingInterceptor extends Interceptor {
  int _maxCharactersPerLine = 200;

  @override
  Future onRequest(RequestOptions options) {
    return super.onRequest(options);
  }

  @override
  Future onResponse(Response response) {
    print(
        "onResponse ${response.statusCode} ${response.request.method} ${response.request.path}");
    String responseAsString = response.data.toString();
    if (responseAsString.length > _maxCharactersPerLine) {
      int iterations =
          (responseAsString.length / _maxCharactersPerLine).floor();
      for (int i = 0; i <= iterations; i++) {
        int endingIndex = i * _maxCharactersPerLine + _maxCharactersPerLine;
        if (endingIndex > responseAsString.length) {
          endingIndex = responseAsString.length;
        }
        print(
            responseAsString.substring(i * _maxCharactersPerLine, endingIndex));
      }
    } else {
      print(response.data);
    }
    print("<-- END HTTP");

    return super.onResponse(response);
  }

  @override
  Future onError(DioError err) {
    print("Error: ${err.error} Message: ${err.message}");
    return super.onError(err);
  }
}
