import 'package:flutter/foundation.dart';
import 'Exceptions/NoConnectionException.dart';
import 'IReponsitory.dart';

class ResponsitoryObject<T> implements IRepository<T> {
  final IRepository<T> source;
  final IRepository<T> cache;
  final Future<bool> Function() hasConnection;
  ResponsitoryObject({
    @required this.source,
    @required this.cache,
    @required this.hasConnection,
  });

  @override
  Future<T> get(dynamic key) async {
    final cacheData = await this.cache.get(key);
    if (cacheData != null) return cacheData;
    if (!await this.hasConnection()) throw NoConnectionException();

    final remoteData = await this.source.get(key);
    this.cache.add(remoteData, key);
    return remoteData;
  }

  @override
  Future<T> add(T object, dynamic key) async {
    if (!await this.hasConnection()) throw NoConnectionException();

    object = await this.source.add(object, key);
    await this.cache.add(object, key);
    return object;
  }

  @override
  Future<void> delete(dynamic id) async {
    if (!await this.hasConnection()) throw NoConnectionException();
    var item = await get(id);
    if (item == null) return;
    await this.source.delete(id);
    await this.cache.delete(id);
  }

  @override
  Future<void> update(key, T object) async {
    if (!await this.hasConnection()) throw NoConnectionException();
    var item = await get(key);
    if (item == null) return;
    await this.source.update(key, object);
    await this.cache.update(key, object);
  }

  @override
  Future<void> clear() async {
    if (!await this.hasConnection()) throw NoConnectionException();
    await this.cache.clear();
  }
}
